-- pNameplates
-- Version:  1.03
-- Authors:  Crushbeers, Tourach23

-- TODO: Setup Move Config

--[[       TABLE OF CONTENTS      ]]--
--[[ ---------------------------- ]]--
--[[ 0. DEFAULT VARIABLE SETTINGS ]]--
--[[ 1. TEXTURES/LOCAL VARIABLES  ]]--
--[[ 2. UTILITY FUNCTIONS		  ]]--
--[[ 3. STYLE FUNCTIONS			  ]]--
--[[ 4. THREAT MANAGEMENT		  ]]--
--[[ 5. UPDATE FUNCTIONS		  ]]--
--[[ 6. CORE FUNCTIONS			  ]]--
--[[ 7. SLASH COMMANDS			  ]]--

--------------------------------------
-- 0. DEFAULT VARIABLE SETTINGS
--------------------------------------

ShowPercent    = true
ShowWhenMax    = true
ThreatEnabled  = true
DPSHealerMode  = true
FontSize_Perc  = 12
FontSize_Name  = 10
FontSize_Level = 12
FontType       = "Fonts\\ARIALN.ttf"

-- Text Element Positions
Mvmt = {
	["perc"]  = { "BOTTOM", "TOP"   , 0, 2 , "THINOUTLINE", "CENTER" },
	["name"]  = { "BOTTOM", "BOTTOM", 0, 2 , "NONE",        "CENTER" },
	["level"] = { "LEFT"  , "RIGHT" , 2, 0 , "THINOUTLINE", "LEFT"   }
}

--------------------------------------
-- 1. TEXUTURES AND LOCAL VARIABLES
--------------------------------------

local Version = "1.03"

local total        = -1
local ForceUpdate  = false
local MoveText     = false
local InCombat     = false
local PVPCombat    = false

local backgroundTexture  	  = "Interface\\Buttons\\WHITE8x8"
local borderDefault      	  = "Interface\\Tooltips\\Nameplate-Border"
local borderTexture      	  = "Interface\\AddOns\\pNameplates\\Textures\\border"
local barTexture         	  = "Interface\\AddOns\\pNameplates\\Textures\\bar"
local React                   = {}
local Players                 = {}

-- Default Unit Colors:
local Colors = {
 -- Key -- { Red, Green, Blue }
	[1]  = { 1.00, 0.00, 0.00 }, -- Hostile
	[2]  = { 1.00, 1.00, 0.00 }, -- Neutral
	[3]  = { 0.00, 1.00, 0.00 }, -- Friendly NPC
	[4]  = { 0.77, 0.12, 0.23 }, -- DK
	[5]  = { 1.00, 0.49, 0.04 }, -- Druid
	[6]  = { 0.67, 0.83, 0.45 }, -- Hunter
	[7]  = { 0.41, 0.80, 0.94 }, -- Mage
	[8]  = { 0.33, 0.54, 0.52 }, -- Monk
	[9]  = { 0.96, 0.55, 0.73 }, -- Paladin
	[10] = { 1.00, 1.00, 1.00 }, -- Priest
	[11] = { 1.00, 0.96, 0.41 }, -- Rogue
	[12] = { 0.00, 0.44, 0.87 }, -- Shaman
	[13] = { 0.58, 0.51, 0.79 }, -- Warlock
	[14] = { 0.78, 0.61, 0.43 }, -- Warrior
	[15] = { 0.00, 0.00, 1.00 }  -- Friendly Player
}

-- Font Variables
local Fonts = {
	["FontType1"] = "ARIALN",
	["FontType2"] = "FRIZQT__",
	["FontType3"] = "MORPHEUS",
	["FontType4"] = "skurri",
}

-- Text Location
local TLoc = {
	["top"]    = { "BOTTOM", "TOP"   ,  0, 2, "THINOUTLINE", "CENTER" },
	["left"]   = { "RIGHT" , "LEFT"  , -2, 0, "THINOUTLINE", "RIGHT"  },
	["right"]  = { "LEFT"  , "RIGHT" ,  2, 0, "THINOUTLINE", "LEFT"   },
	["center"] = { "BOTTOM", "BOTTOM",  0, 2, "NONE",        "CENTER" }
}

-- Threat Setting Lists
local ThreatLow = {
	Right   = { loc = "TOPRIGHT", x = 1, y = 2 },
	Left    = { loc = "BOTTOMLEFT", x = -1, y = -2 },
	Color   = { r = 0.0, g = 0.7, b = 0.0 },
	Texture = "Interface\\AddOns\\pNameplates\\Textures\\threat_LOW",
	HPScale = 1,
}

local ThreatMed = {
	Right   = { loc = "TOPRIGHT", x = 4, y = 5 },
	Left    = { loc = "BOTTOMLEFT", x = -4, y = -5 },
	Color   = { r = 0.7, g = 0.7, b = 0.0 },
	Texture = "Interface\\AddOns\\pNameplates\\Textures\\threat_MED",
	HPScale = 1,
}

local ThreatHigh = {
	Right   = { loc = "TOPRIGHT", x = 4, y = 5 },
	Left    = { loc = "BOTTOMLEFT", x = -4, y = -5 },
	Color   = { r = 0.8, g = 0.0, b = 0.0 },
	Texture = "Interface\\AddOns\\pNameplates\\Textures\\threat_HIGH",
	HPScale = 1.18,
}

--------------------------------------
-- 2. UTILITY FUNCTIONS
--------------------------------------

local function RoundNum(num, dec)
	num = num * math.pow(10, dec)
	num = math.ceil(num)
	num = num / math.pow(10, dec)
	return num
end

local function ParseString(s)
	local loc = strlen(s) - 4
	return string.sub(s, 7, loc)
end

local function AlignObject(object)
	local _, anchor, _, x, y = object:GetPoint()
	object:ClearAllPoints()
	object:SetPoint("BOTTOM", anchor, "BOTTOM", x, y)
end

local function IsNameplate(object)
    local bo = select(2, object:GetRegions())
    if (bo and bo:GetTexture() == borderTexture) then return true end
    if (not bo or bo:GetObjectType() ~= "Texture" or bo:GetTexture() ~= borderDefault) then
      	object.Styled = true
      	return false
    end
    return true
end

--------------------------------------
-- 3. STYLE FUNCTIONS
--------------------------------------

local function StyleTextures(plate)
	if (not plate.Highlight) then return end
	plate.Highlight:SetTexture(1, 1, 1, 0.2)
	plate.Highlight:SetPoint("TOPLEFT", plate.Border, "TOPLEFT", 1, -1)
	plate.Highlight:SetPoint("BOTTOMRIGHT", plate.Border, "BOTTOMRIGHT", -1, 1)
	plate.OldLevel:Hide()
end

local function HideBlizz(plate)
	plate.OldName:Hide()
	plate.Threat:SetTexture(0, 0, 0, 0)
	plate.Dragon:SetTexture(0, 0, 0, 0)
    plate.Castbar.Shield:SetTexture(0, 0, 0, 0)
end

local function StyleCastbar(plate)
	plate.Castbar:SetBackdrop({
        bgFile = backgroundTexture,
        insets = { left = -1, right = -1, top = -1, bottom = -1 }
    })
    plate.Castbar:SetBackdropColor(0, 0, 0, 0.6)
    plate.Castbar:ClearAllPoints()
    plate.Castbar:SetPoint("TOPRIGHT", plate.Health, "BOTTOMRIGHT", 0, -4)
    plate.Castbar:SetPoint("BOTTOMLEFT", plate.Health, "BOTTOMLEFT", 0, -15)
	
	plate.Castbar.Border:SetTexture(0, 0, 0, 1)
    plate.Castbar.Border:SetTexCoord(0, 1, 0, 1)
    plate.Castbar.Border:ClearAllPoints()
    plate.Castbar.Border:SetPoint("TOPRIGHT", plate.Castbar, 0, 0)
    plate.Castbar.Border:SetPoint("BOTTOMLEFT", plate.Castbar, 0, 0)
    plate.Castbar.Border:SetDrawLayer("BORDER")
    
    plate.Castbar.Fill:SetTexture(barTexture)
    plate.Castbar.Fill:ClearAllPoints()
    plate.Castbar.Fill:SetPoint("TOPRIGHT", plate.Castbar.Border, "TOPRIGHT", 1, -1)
    plate.Castbar.Fill:SetPoint("TOPRIGHT", plate.Castbar.Border, "BOTTOMLEFT", -1, 1)
    
    plate.Castbar.Icon:SetSize(12,12)
    plate.Castbar.Icon:ClearAllPoints()
    plate.Castbar.Icon:SetPoint("LEFT", plate.Castbar, "RIGHT", 2, 0)
    
    plate.Castbar.Time = plate.Castbar:CreateFontString(nil, "HIGH")
	plate.Castbar.Time:SetFont(FontType, 10, "THINOUTLINE")
	plate.Castbar.Time:SetPoint("RIGHT", 0, 0)
	
	plate.Castbar.Spell = plate.Castbar:CreateFontString(nil, "HIGH")
	plate.Castbar.Spell:SetFont(FontType, 10, "THINOUTLINE")
	plate.Castbar.Spell:SetPoint("LEFT", 1, 0)
end

local function StyleFonts(plate)
	plate.Health.hpval:SetFont(FontType, FontSize_Perc, Mvmt["perc"][5])
	plate.Name:SetFont(FontType, FontSize_Name, Mvmt["name"][5])
	plate.Level:SetFont(FontType, FontSize_Level, Mvmt["level"][5])
end

local function ApplyShadow(text)
	text:SetShadowColor(0, 0, 0, 1)
	text:SetShadowOffset(1, -1)
end

local function RemoveShadow(text)
	text:SetShadowColor(0, 0, 0, 0)
end

local function StyleLevelText(plate)
	plate.Level:SetTextColor(plate.OldLevel:GetTextColor())
	if (plate.Boss:IsShown()) then plate.Level:Hide()
	else plate.Level:Show() end
	
	local string = plate.OldLevel:GetText()
	if (not string) then string = "" end
	return string
end

--------------------------------------
-- 4. THREAT MANAGEMENT
--------------------------------------

local function AssignDefaultColor(plate)
	local r = RoundNum(select(1, plate.Health:GetStatusBarColor()), 2)
	local g = RoundNum(select(2, plate.Health:GetStatusBarColor()), 2)
	local b = RoundNum(select(3, plate.Health:GetStatusBarColor()), 2)
	
	Players[plate.Text] = false -- Initialize to false
	
	for i,v in ipairs(Colors) do
		if (r == v[1] and g == v[2] and b == v[3]) then
			React[plate.Text] = v
			if (i > 3) then Players[plate.Text] = true end
			break
		end
	end
end

local function SetThreatStyle(plate, Threat)	
	plate.ThreatBorder:SetPoint(Threat.Right.loc, plate.Border, Threat.Right.x, Threat.Right.y)
	plate.ThreatBorder:SetPoint(Threat.Left.loc, plate.Border, Threat.Left.x, Threat.Left.y)
	plate.ThreatBorder:SetTexture(Threat.Texture)
	plate.Health:SetStatusBarColor(Threat.Color.r, Threat.Color.g, Threat.Color.b, 1)
	plate.Health:SetScale(Threat.HPScale)
end

local function UpdateThreat(plate)
	if (Players[plate.Text] == nil) then AssignDefaultColor(plate) end
	if (not InCombat and plate.ThreatBorder:IsShown()) or (Players[plate.Text] or not ThreatEnabled) then
		plate.ThreatBorder:Hide()
		plate.Health:SetStatusBarColor(React[plate.Text][1], React[plate.Text][2], React[plate.Text][3], 1)
		plate.Health:SetScale(1)
		return 
	elseif (InCombat) then
		plate.ThreatBorder:Show()
	else return end -- Returns if out of combat AND threat objects are hidden
	
	local red, green, blue, _ = plate.Threat:GetVertexColor()
	local shown = plate.Threat:IsShown()
	
	if (not shown) then 
		if (DPSHealerMode) then SetThreatStyle(plate, ThreatLow)
		else SetThreatStyle(plate, ThreatHigh) end
	elseif (red > 0) then
		if (green > 0) then SetThreatStyle(plate, ThreatMed) return end
		if (DPSHealerMode) then SetThreatStyle(plate, ThreatHigh)
		else SetThreatStyle(plate, ThreatLow) end
	end
end

--------------------------------------
-- 5. UPDATE FUNCTIONS
--------------------------------------

local function UpdateDisplay(plate)
	plate.Text = plate.OldName:GetText()
	if (ThreatEnabled or ForceUpdate) then UpdateThreat(plate) end
	
	plate.Health:SetHeight(14)
	plate.Name:SetText(plate.Text)
	plate.Level:SetText(StyleLevelText(plate))
	
	if (ShowPercent) then
		local _, max = plate.Health:GetMinMaxValues()
		local value = plate.Health:GetValue()
		local percent = math.floor((value/max)*100)
		if (percent >= 100 and not ShowWhenMax) then plate.Health.hpval:SetText("")
		else plate.Health.hpval:SetText(percent.."%") end
	else
		plate.Health.hpval:SetText("")
	end
end

local function UpdateCastInfo(Castbar)
	local SpellName = select(3, UnitCastingInfo("target"))
	if (not SpellName) then SpellName = select(3, UnitChannelInfo("target")) end
	local _, max = Castbar:GetMinMaxValues()
	local current = Castbar:GetValue()
	local final = math.floor((max-current)*10)/10
	
	Castbar.Time:SetText(final)
	Castbar.Spell:SetText(SpellName)
	
	-- Fixes phantom castbar frames appearing
	if (not final or final <= 0) then Castbar:Hide()
	else Castbar:Show() end
end

local function UpdateTextLocations(plate)
	plate.Health.hpval:ClearAllPoints()
	plate.Health.hpval:SetPoint(Mvmt["perc"][1], plate.Health, Mvmt["perc"][2], Mvmt["perc"][3], Mvmt["perc"][4])
	plate.Health.hpval:SetJustifyH(Mvmt["perc"][6])
	if (Mvmt["perc"][5] == "NONE") then ApplyShadow(plate.Health.hpval)
	else RemoveShadow(plate.Health.hpval) end
	
	plate.Name:ClearAllPoints()
	plate.Name:SetPoint(Mvmt["name"][1], plate.Health, Mvmt["name"][2], Mvmt["name"][3], Mvmt["name"][4])
	plate.Name:SetJustifyH(Mvmt["name"][6])
	if (Mvmt["name"][5] == "NONE") then ApplyShadow(plate.Name)
	else RemoveShadow(plate.Name) end
	
	plate.Level:ClearAllPoints()
	plate.Level:SetPoint(Mvmt["level"][1], plate.Health, Mvmt["level"][2], Mvmt["level"][3], Mvmt["level"][4])
	plate.Level:SetJustifyH(Mvmt["level"][6])
	if (Mvmt["level"][5] == "NONE") then ApplyShadow(plate.Level)
	else RemoveShadow(plate.Level) end
	
	plate.Boss:ClearAllPoints()
	plate.Boss:SetPoint(plate.Level:GetPoint())
end

--------------------------------------
-- 6. CORE FUNCTIONS
--------------------------------------

local function init(plate)
	plate:SetToplevel(true)

	plate.Health, plate.Castbar = plate:GetChildren()
    plate.Castbar.Fill, plate.Castbar.Border, plate.Castbar.Shield, plate.Castbar.Icon = plate.Castbar:GetRegions()
    plate.Threat, plate.Border, plate.Highlight, plate.OldName, plate.OldLevel, plate.Boss, _, plate.Dragon = plate:GetRegions()

	plate.Border:SetTexture(borderTexture)
    plate.Border:SetVertexColor(0, 0, 0)
    plate.Border:ClearAllPoints()
    plate.Border:SetPoint("TOPRIGHT", plate.Health, 1, 1)
    plate.Border:SetPoint("BOTTOMLEFT", plate.Health, -1, -1)
    
    plate.ThreatBorder = plate:CreateTexture(nil, "Texture")
    plate.ThreatBorder:SetPoint("TOPRIGHT", plate.Border, 1, 2)
    plate.ThreatBorder:SetPoint("BOTTOMLEFT", plate.Border, -1, -2)
    
	plate.Health:SetHeight(14)
	plate.Health:SetStatusBarTexture(barTexture)
    plate.Health:SetBackdrop({
        bgFile = backgroundTexture,
        insets = { left = 0, right = 0, top = 0, bottom = 0 }
    })
    plate.Health:SetBackdropColor(0, 0, 0, 0.6)
	
	plate.Level = plate:CreateFontString(nil, "HIGH")
	plate.Level:SetFont(FontType, FontSize_Level, Mvmt["level"][5])
	
	plate.Name = plate:CreateFontString(nil, "HIGH")
	plate.Name:SetFont(FontType, FontSize_Name, Mvmt["name"][5])
	plate.Name:SetText(plate.OldName:GetText())
	plate.Name:SetWordWrap(false)
	plate.Name:SetWidth(100)
	
	plate.Health.hpval = plate.Health:CreateFontString(nil, "HIGH")
	plate.Health.hpval:SetFont(FontType, FontSize_Perc, Mvmt["perc"][5])
	
	-- Initialization Stuff
	plate.Mod = true
	UpdateTextLocations(plate)
	UpdateDisplay(plate)
	StyleTextures(plate)
	AlignObject(plate.Health)
	StyleCastbar(plate)
	HideBlizz(plate)
	
	plate.Castbar:SetScript("OnValueChanged", UpdateCastInfo)
	
	plate.Health:SetScript("OnShow", function()
		UpdateDisplay(plate)
		StyleTextures(plate)
		UpdateCastInfo(plate.Castbar)
		AlignObject(plate.Health)
	end)
	plate.Health:SetScript("OnHide", function() UpdateDisplay(plate) end)
	plate.Health:SetScript("OnValueChanged", function() UpdateDisplay(plate) end)
	plate.Health:SetScript("OnMinMaxChanged", function() UpdateDisplay(plate) end)
end

local function GetObjects(plate, elapsed)
    local current = select("#", WorldFrame:GetChildren())
    if (current ~= total or ForceUpdate or MoveText) then
    	total = current
	  	for i = 1, current do
			local plate = select(i, WorldFrame:GetChildren())
			if (IsNameplate(plate)) then
		  		if (not plate.Mod) then init(plate)
		  		else 
		  			UpdateDisplay(plate)
		  			StyleTextures(plate)
		  			if (ForceUpdate or MoveText) then StyleFonts(plate) end
		  			if (MoveText) then UpdateTextLocations(plate) end
		  		end
		  	end
		end
		ForceUpdate = false
		MoveText = false
    end
end

local f = CreateFrame("Frame")
f:SetScript("OnEvent", function(plate, event)
	ForceUpdate = true
	if (event == "PLAYER_LOGIN") then
		SetCVar("ShowClassColorInNameplate", 1)
		plate:SetScript("OnUpdate", GetObjects)
	elseif (event == "PLAYER_REGEN_DISABLED") then
		InCombat = true
	elseif (event == "PLAYER_REGEN_ENABLED") then
		InCombat = false
	end
	collectgarbage()
end)
  
f:RegisterEvent("PLAYER_LOGIN")
f:RegisterEvent("PLAYER_REGEN_DISABLED") -- Enter Combat
f:RegisterEvent("PLAYER_REGEN_ENABLED")  -- Leave Combat

--------------------------------------
-- 7. SLASH COMMANDS
--------------------------------------

local function handler(args)
	local cur = nil
	local cmd = string.split(' ', args):lower() or args:lower()
	if (cmd:match"font") then
		cmd = select(2, string.split(' ', args))
		if (not cmd) then cmd = "showcommands" end
		if (cmd:match"size") then
			local NewSize = nil
			cmd = select(3, string.split(' ', args))
			if (not cmd) then cmd = "showcommands"
			else NewSize = (select(4, string.split(' ', args)) + 0) end
			if (cmd:match"perc") then 
				FontSize_Perc = NewSize
				print("|cFF6EE1E6p|rNameplates |cFF6EE1E6Percent Font size set to|r "..FontSize_Perc.."|cFF6EE1E6.|r")
			elseif (cmd:match"name") then
				FontSize_Name = NewSize
				print("|cFF6EE1E6p|rNameplates |cFF6EE1E6Name Font size set to|r "..FontSize_Name.."|cFF6EE1E6.|r")
			elseif (cmd:match"level") then
				FontSize_Level = NewSize
				print("|cFF6EE1E6p|rNameplates |cFF6EE1E6Level Font size set to|r "..FontSize_Level.."|cFF6EE1E6.|r")
			else
				print("|cFF6EE1E6p|rNameplates |cFF6EE1E6Font Size commands:|r")
				print("|cFF6EE1E6/pp font size perc ##|r, to set Percent Font Size.")
				print("|cFF6EE1E6/pp font size name ##|r, to set Unit Name Font Size.")
				print("|cFF6EE1E6/pp font size level ##|r, to set Unit Level Font Size.")
				print("|cFFCB4040Note|r: If you enter a non-number, you will get an LUA error.")
			end
		elseif (cmd:match"type") then
			cmd = select(3, string.split(' ', args))
			if (not cmd) then
				print("|cFF6EE1E6p|rNameplates |cFF6EE1E6Font Type commands:|r ")
				print("|cFF6EE1E6/pp font type #|r, (1-4) to select Font Type.")
				print("|cFF6EE1E6Font Types|r:")
				print("|cFF6EE1E61|r. "..Fonts["FontType1"])
				print("|cFF6EE1E62|r. "..Fonts["FontType2"])
				print("|cFF6EE1E63|r. "..Fonts["FontType3"])
				print("|cFF6EE1E64|r. "..Fonts["FontType4"])
			elseif (not cmd:match"1" and not cmd:match"2" and not cmd:match"3" and not cmd:match"4") then
				print("|cFFCB4040Invalid Entry|r: |cFF6EE1E6/pp font type|r, to see Font Type options again.")
			else
				FontType = "Fonts\\"..Fonts["FontType"..cmd]..".ttf"
				print("|cFF6EE1E6p|rNameplates |cFF6EE1E6Font type changed to |r"..Fonts["FontType"..cmd].."|cFF6EE1E6.|r")
			end
		else
			cur = ParseString(FontType)
			print("|cFF6EE1E6p|rNameplates |cFF6EE1E6Font commands:|r  (Current Setting: "..cur..")")
			print("|cFF6EE1E6/pp font size|r, to see Font Size options.")
			print("|cFF6EE1E6/pp font type|r, to see Font Size options.")
		end
	
	elseif (cmd:match"move") then
		cmd = select(2, string.split(' ', args))
		if (not cmd) then
			print("|cFF6EE1E6p|rNameplates |cFF6EE1E6Movement commands:|r ")
			print("|cFF6EE1E6Valid Locations|r: top, left, right, center")
    		print("|cFF6EE1E6/pp move perc 'location'|r, to relocate Percent Text.")
    		print("|cFF6EE1E6/pp move name 'location'|r, to relocate Name Text.")
    		print("|cFF6EE1E6/pp move level 'location'|r, to relocate Level Text.")
    	else
    		cur = cmd
    		cmd = select(3, string.split(' ', args))
			if (not cmd) then cmd = "error" end
			if (cmd:match"top" or cmd:match"left" or cmd:match"right" or cmd:match"center") then
				Mvmt[cur] = TLoc[cmd]
				MoveText = true
			else print("|cFFCB4040Invalid Entry|r: |cFF6EE1E6/pp move|r, to see Movement options again.")
			end
    	end
    elseif (cmd:match"percent") then
    	if (ShowPercent) then
			ShowPercent = false
			print("|cFF6EE1E6p|rNameplates|cFF6EE1E6: Health percent disabled.|r")
		else
			ShowPercent = true
			print("|cFF6EE1E6p|rNameplates|cFF6EE1E6: Health percent enabled.|r")
		end
    elseif (cmd:match"max") then
    	if (ShowWhenMax) then 
    		ShowWhenMax = false
    		print("|cFF6EE1E6p|rNameplates|cFF6EE1E6: 100% Health display disabled.|r")
    	else
    		ShowWhenMax = true
    		print("|cFF6EE1E6p|rNameplates|cFF6EE1E6: 100% Health display enabled.|r")
    	end
    elseif (cmd:match"threat") then
    	if (ThreatEnabled) then 
    		ThreatEnabled = false
    		print("|cFF6EE1E6p|rNameplates|cFF6EE1E6: Threat coloring disabled.|r")
    	else
    		ThreatEnabled = true
    		print("|cFF6EE1E6p|rNameplates|cFF6EE1E6: Threat coloring enabled.|r")
    	end
    elseif (cmd:match"role") then
    	cmd = select(2, string.split(' ', args))
    	if (not cmd) then
    		if (DPSHealerMode) then cur = "DPS/Heal mode"
    		else cur = "Tank mode" end
    		print("|cFF6EE1E6p|rNameplates |cFF6EE1E6Role commands:|r  (Current Setting: "..cur..")")
    		print("|cFF6EE1E6/pp role tank|r, for Tank mode.")
    		print("|cFF6EE1E6/pp role dps OR heal|r, for DPS/Healing mode.")
    	elseif (cmd:match"tank") then
    		DPSHealerMode = false
    		print("|cFF6EE1E6p|rNameplates|cFF6EE1E6: Tank mode enabled.|r")
    	elseif (cmd:match"dps" or cmd:match"heal") then
    		DPSHealerMode = true
    		print("|cFF6EE1E6p|rNameplates|cFF6EE1E6: DPS/Healing mode enabled.|r")
    	else
    		print("|cFFCB4040Invalid Entry|r: |cFF6EE1E6/pp role|r, to see Role options again.")
    	end
    elseif (cmd:match"motion") then
    	cmd = select(2, string.split(' ', args))
    	if (not cmd) then
    		cur = GetCVar("nameplateMotion")
    		print("|cFF6EE1E6p|rNameplates |cFF6EE1E6Motion commands:|r  (Current Setting: "..cur..")")
    		print("|cFF6EE1E6/pp motion 0|r, for Overlap.")
    		print("|cFF6EE1E6/pp motion 1|r, for Stacking.")
    		print("|cFF6EE1E6/pp motion 2|r, for Spreading.")
    	elseif (cmd:match"0") then SetCVar("nameplateMotion", 0)
    	elseif (cmd:match"1") then SetCVar("nameplateMotion", 1)
    	elseif (cmd:match"2") then SetCVar("nameplateMotion", 2)
    	else print("|cFFCB4040Invalid Entry|r: |cFF6EE1E6/pp motion|r, to see Motion options again.") end
    else
		print("|cFF6EE1E6p|rNameplates |cFF6EE1E6Command List:|r")
		print("|cFF6EE1E6/pp percent|r, to toggle Health Percents.")
		print("|cFF6EE1E6/pp max|r, to toggle 100% Health.")
		print("|cFF6EE1E6/pp font|r, to see Font options.")
		print("|cFF6EE1E6/pp move|r, to see Text Location options.")
		print("|cFF6EE1E6/pp role|r, to see Threat Role options.")
		print("|cFF6EE1E6/pp threat|r, to toggle Threat Coloring.")
		print("|cFF6EE1E6/pp motion|r, to see Nameplate Motion options.")
    end
    ForceUpdate = true
end
 
SLASH_pp1 = "/pp";
SlashCmdList["pp"] = handler;

print("|cFF6EE1E6p|rNameplates  |cFF6EE1E6(v"..Version..")|r  is loaded: |cFF6EE1E6/pp|r for help.")